@@ -1,25 +1,27 @@
-# -*- coding: utf-8 -*-
 """Event loop implementation."""
-from __future__ import absolute_import, unicode_literals
+
+from __future__ import annotations
 
 import errno
+import threading
 from contextlib import contextmanager
+from queue import Empty
 from time import sleep
-from types import GeneratorType as generator  # noqa
+from types import GeneratorType as generator
+
+from vine import Thenable, promise
 
-from kombu.five import Empty, python_2_unicode_compatible, range
 from kombu.log import get_logger
 from kombu.utils.compat import fileno
 from kombu.utils.eventio import ERR, READ, WRITE, poll
 from kombu.utils.objects import cached_property
-from vine import Thenable, promise
 
 from .timer import Timer
 
 __all__ = ('Hub', 'get_event_loop', 'set_event_loop')
 logger = get_logger(__name__)
 
-_current_loop = None
+_current_loop: Hub | None = None
 
 W_UNKNOWN_EVENT = """\
 Received unknown event %r for fd %r, please contact support!\
@@ -39,23 +41,23 @@ def _dummy_context(*args, **kwargs):
     yield
 
 
-def get_event_loop():
+def get_event_loop() -> Hub | None:
     """Get current event loop object."""
     return _current_loop
 
 
-def set_event_loop(loop):
+def set_event_loop(loop: Hub | None) -> Hub | None:
     """Set the current event loop object."""
     global _current_loop
     _current_loop = loop
     return loop
 
 
-@python_2_unicode_compatible
-class Hub(object):
+class Hub:
     """Event loop object.
 
     Arguments:
+    ---------
         timer (kombu.asynchronous.Timer): Specify custom timer instance.
     """
 
@@ -80,6 +82,7 @@ class Hub(object):
         self.on_tick = set()
         self.on_close = set()
         self._ready = set()
+        self._ready_lock = threading.Lock()
 
         self._running = False
         self._loop = None
@@ -128,7 +131,7 @@ class Hub(object):
         self.call_soon(_raise_stop_error)
 
     def __repr__(self):
-        return '<Hub@{0:#x}: R:{1} W:{2}>'.format(
+        return '<Hub@{:#x}: R:{} W:{}>'.format(
             id(self), len(self.readers), len(self.writers),
         )
 
@@ -200,7 +203,8 @@ class Hub(object):
     def call_soon(self, callback, *args):
         if not isinstance(callback, Thenable):
             callback = promise(callback, args)
-        self._ready.add(callback)
+        with self._ready_lock:
+            self._ready.add(callback)
         return callback
 
     def call_later(self, delay, callback, *args):
@@ -244,6 +248,12 @@ class Hub(object):
         except (AttributeError, KeyError, OSError):
             pass
 
+    def _pop_ready(self):
+        with self._ready_lock:
+            ready = self._ready
+            self._ready = set()
+            return ready
+
     def close(self, *args):
         [self._unregister(fd) for fd in self.readers]
         self.readers.clear()
@@ -259,8 +269,7 @@ class Hub(object):
         # To avoid infinite loop where one of the callables adds items
         # to self._ready (via call_soon or otherwise).
         # we create new list with current self._ready
-        todos = list(self._ready)
-        self._ready = set()
+        todos = self._pop_ready()
         for item in todos:
             item()
 
@@ -290,17 +299,17 @@ class Hub(object):
         propagate = self.propagate_errors
 
         while 1:
-            todo = self._ready
-            self._ready = set()
-
-            for tick_callback in on_tick:
-                tick_callback()
+            todo = self._pop_ready()
 
             for item in todo:
                 if item:
                     item()
 
             poll_timeout = fire_timers(propagate=propagate) if scheduled else 1
+
+            for tick_callback in on_tick:
+                tick_callback()
+
             #  print('[[[HUB]]]: %s' % (self.repr_active(),))
             if readers or writers:
                 to_consolidate = []
