@@ -1,3 +1,5 @@
+from __future__ import annotations
+
 import datetime
 from queue import Empty
 from unittest.mock import MagicMock, call, patch
@@ -8,6 +10,15 @@ from kombu import Connection
 
 pymongo = pytest.importorskip('pymongo')
 
+# must import following after above validation to avoid error
+# and skip tests if missing
+# these are used to define real spec of the corresponding mocks,
+# to ensure called methods exist in real objects
+# pylint: disable=C0413
+from pymongo.collection import Collection  # isort:skip # noqa: E402
+from pymongo.database import Database  # isort:skip # noqa: E402
+from kombu.transport.mongodb import BroadcastCursor  # isort:skip # noqa: E402
+
 
 def _create_mock_connection(url='', **kwargs):
     from kombu.transport import mongodb
@@ -20,7 +31,9 @@ def _create_mock_connection(url='', **kwargs):
         now = datetime.datetime.utcnow()
 
         def _create_client(self):
-            mock = MagicMock(name='client')
+            # not really a 'MongoClient',
+            # but an actual pre-established Database connection
+            mock = MagicMock(name='client', spec=Database)
 
             # we need new mock object for every collection
             def get_collection(name):
@@ -28,7 +41,9 @@ def _create_mock_connection(url='', **kwargs):
                     return self.collections[name]
                 except KeyError:
                     mock = self.collections[name] = MagicMock(
-                        name='collection:%s' % name)
+                        name='collection:%s' % name,
+                        spec=Collection,
+                    )
 
                     return mock
 
@@ -106,7 +121,7 @@ class BaseMongoDBChannelCase:
         else:
             method.side_effect = values
 
-    def declare_droadcast_queue(self, queue):
+    def declare_broadcast_queue(self, queue):
         self.channel.exchange_declare('fanout_exchange', type='fanout')
 
         self.channel._queue_bind('fanout_exchange', 'foo', '*', queue)
@@ -117,9 +132,9 @@ class BaseMongoDBChannelCase:
         return self.channel._broadcast_cursors[queue]
 
     def set_broadcast_return_value(self, queue, *values):
-        self.declare_droadcast_queue(queue)
+        self.declare_broadcast_queue(queue)
 
-        cursor = MagicMock(name='cursor')
+        cursor = MagicMock(name='cursor', spec=BroadcastCursor)
         cursor.__iter__.return_value = iter(values)
 
         self.channel._broadcast_cursors[queue]._cursor = iter(cursor)
@@ -151,16 +166,15 @@ class test_mongodb_channel(BaseMongoDBChannelCase):
 
     def test_get(self):
 
-        self.set_operation_return_value('messages', 'find_and_modify', {
+        self.set_operation_return_value('messages', 'find_one_and_delete', {
             '_id': 'docId', 'payload': '{"some": "data"}',
         })
 
         event = self.channel._get('foobar')
         self.assert_collection_accessed('messages')
         self.assert_operation_called_with(
-            'messages', 'find_and_modify',
-            query={'queue': 'foobar'},
-            remove=True,
+            'messages', 'find_one_and_delete',
+            {'queue': 'foobar'},
             sort=[
                 ('priority', pymongo.ASCENDING),
             ],
@@ -168,7 +182,11 @@ class test_mongodb_channel(BaseMongoDBChannelCase):
 
         assert event == {'some': 'data'}
 
-        self.set_operation_return_value('messages', 'find_and_modify', None)
+        self.set_operation_return_value(
+            'messages',
+            'find_one_and_delete',
+            None,
+        )
         with pytest.raises(Empty):
             self.channel._get('foobar')
 
@@ -188,37 +206,37 @@ class test_mongodb_channel(BaseMongoDBChannelCase):
         self.channel._put('foobar', {'some': 'data'})
 
         self.assert_collection_accessed('messages')
-        self.assert_operation_called_with('messages', 'insert', {
+        self.assert_operation_called_with('messages', 'insert_one', {
             'queue': 'foobar',
             'priority': 9,
             'payload': '{"some": "data"}',
         })
 
     def test_put_fanout(self):
-        self.declare_droadcast_queue('foobar')
+        self.declare_broadcast_queue('foobar')
 
         self.channel._put_fanout('foobar', {'some': 'data'}, 'foo')
 
         self.assert_collection_accessed('messages.broadcast')
-        self.assert_operation_called_with('broadcast', 'insert', {
+        self.assert_operation_called_with('broadcast', 'insert_one', {
             'queue': 'foobar', 'payload': '{"some": "data"}',
         })
 
     def test_size(self):
-        self.set_operation_return_value('messages', 'find.count', 77)
+        self.set_operation_return_value('messages', 'count_documents', 77)
 
         result = self.channel._size('foobar')
         self.assert_collection_accessed('messages')
         self.assert_operation_called_with(
-            'messages', 'find', {'queue': 'foobar'},
+            'messages', 'count_documents', {'queue': 'foobar'},
         )
 
         assert result == 77
 
     def test_size_fanout(self):
-        self.declare_droadcast_queue('foobar')
+        self.declare_broadcast_queue('foobar')
 
-        cursor = MagicMock(name='cursor')
+        cursor = MagicMock(name='cursor', spec=BroadcastCursor)
         cursor.get_size.return_value = 77
         self.channel._broadcast_cursors['foobar'] = cursor
 
@@ -227,20 +245,20 @@ class test_mongodb_channel(BaseMongoDBChannelCase):
         assert result == 77
 
     def test_purge(self):
-        self.set_operation_return_value('messages', 'find.count', 77)
+        self.set_operation_return_value('messages', 'count_documents', 77)
 
         result = self.channel._purge('foobar')
         self.assert_collection_accessed('messages')
         self.assert_operation_called_with(
-            'messages', 'remove', {'queue': 'foobar'},
+            'messages', 'delete_many', {'queue': 'foobar'},
         )
 
         assert result == 77
 
     def test_purge_fanout(self):
-        self.declare_droadcast_queue('foobar')
+        self.declare_broadcast_queue('foobar')
 
-        cursor = MagicMock(name='cursor')
+        cursor = MagicMock(name='cursor', spec=BroadcastCursor)
         cursor.get_size.return_value = 77
         self.channel._broadcast_cursors['foobar'] = cursor
 
@@ -276,11 +294,11 @@ class test_mongodb_channel(BaseMongoDBChannelCase):
         self.channel._queue_bind('test_exchange', 'foo', '*', 'foo')
         self.assert_collection_accessed('messages.routing')
         self.assert_operation_called_with(
-            'routing', 'update',
-            {'queue': 'foo', 'pattern': '*',
-             'routing_key': 'foo', 'exchange': 'test_exchange'},
+            'routing', 'update_one',
             {'queue': 'foo', 'pattern': '*',
              'routing_key': 'foo', 'exchange': 'test_exchange'},
+            {'$set': {'queue': 'foo', 'pattern': '*',
+             'routing_key': 'foo', 'exchange': 'test_exchange'}},
             upsert=True,
         )
 
@@ -288,13 +306,13 @@ class test_mongodb_channel(BaseMongoDBChannelCase):
         self.channel.queue_delete('foobar')
         self.assert_collection_accessed('messages.routing')
         self.assert_operation_called_with(
-            'routing', 'remove', {'queue': 'foobar'},
+            'routing', 'delete_many', {'queue': 'foobar'},
         )
 
     def test_queue_delete_fanout(self):
-        self.declare_droadcast_queue('foobar')
+        self.declare_broadcast_queue('foobar')
 
-        cursor = MagicMock(name='cursor')
+        cursor = MagicMock(name='cursor', spec=BroadcastCursor)
         self.channel._broadcast_cursors['foobar'] = cursor
 
         self.channel.queue_delete('foobar')
@@ -313,20 +331,42 @@ class test_mongodb_channel(BaseMongoDBChannelCase):
             'messages.broadcast', capped=True, size=100000,
         )
 
+    def test_create_broadcast_exists(self):
+        # simulate already created collection
+        self.channel.client.list_collection_names.return_value = [
+            'messages.broadcast'
+        ]
+
+        broadcast = self.channel._create_broadcast(self.channel.client)
+        self.channel.client.create_collection.assert_not_called()
+        assert broadcast is None  # not returned since not created
+
+    def test_get_broadcast_cursor_created(self):
+        self.channel._fanout_queues['foobar'] = 'fanout_exchange'
+        created_cursor = self.channel._get_broadcast_cursor('foobar')
+        cached_cursor = self.channel._broadcast_cursors['foobar']
+        assert cached_cursor is created_cursor
+
+    def test_get_broadcast_cursor_exists(self):
+        self.declare_broadcast_queue('foobar')
+        cached_cursor = self.channel._broadcast_cursors['foobar']
+        getter_cursor = self.channel._get_broadcast_cursor('foobar')
+        assert cached_cursor is getter_cursor
+
     def test_ensure_indexes(self):
         self.channel._ensure_indexes(self.channel.client)
 
         self.assert_operation_called_with(
-            'messages', 'ensure_index',
+            'messages', 'create_index',
             [('queue', 1), ('priority', 1), ('_id', 1)],
             background=True,
         )
         self.assert_operation_called_with(
-            'broadcast', 'ensure_index',
+            'broadcast', 'create_index',
             [('queue', 1)],
         )
         self.assert_operation_called_with(
-            'routing', 'ensure_index', [('queue', 1), ('exchange', 1)],
+            'routing', 'create_index', [('queue', 1), ('exchange', 1)],
         )
 
     def test_create_broadcast_cursor(self):
@@ -381,9 +421,9 @@ class test_mongodb_channel_ttl(BaseMongoDBChannelCase):
         self.channel._new_queue('foobar')
 
         self.assert_operation_called_with(
-            'queues', 'update',
+            'queues', 'update_one',
             {'_id': 'foobar'},
-            {'_id': 'foobar', 'options': {}, 'expire_at': None},
+            {'$set': {'_id': 'foobar', 'options': {}, 'expire_at': None}},
             upsert=True,
         )
 
@@ -393,25 +433,23 @@ class test_mongodb_channel_ttl(BaseMongoDBChannelCase):
             '_id': 'docId', 'options': {'arguments': {'x-expires': 777}},
         })
 
-        self.set_operation_return_value('messages', 'find_and_modify', {
+        self.set_operation_return_value('messages', 'find_one_and_delete', {
             '_id': 'docId', 'payload': '{"some": "data"}',
         })
 
         self.channel._get('foobar')
         self.assert_collection_accessed('messages', 'messages.queues')
         self.assert_operation_called_with(
-            'messages', 'find_and_modify',
-            query={'queue': 'foobar'},
-            remove=True,
+            'messages', 'find_one_and_delete',
+            {'queue': 'foobar'},
             sort=[
                 ('priority', pymongo.ASCENDING),
             ],
         )
         self.assert_operation_called_with(
-            'routing', 'update',
+            'routing', 'update_many',
             {'queue': 'foobar'},
             {'$set': {'expire_at': self.expire_at}},
-            multi=True,
         )
 
     def test_put(self):
@@ -422,7 +460,7 @@ class test_mongodb_channel_ttl(BaseMongoDBChannelCase):
         self.channel._put('foobar', {'some': 'data'})
 
         self.assert_collection_accessed('messages')
-        self.assert_operation_called_with('messages', 'insert', {
+        self.assert_operation_called_with('messages', 'insert_one', {
             'queue': 'foobar',
             'priority': 9,
             'payload': '{"some": "data"}',
@@ -437,12 +475,14 @@ class test_mongodb_channel_ttl(BaseMongoDBChannelCase):
         self.channel._queue_bind('test_exchange', 'foo', '*', 'foo')
         self.assert_collection_accessed('messages.routing')
         self.assert_operation_called_with(
-            'routing', 'update',
+            'routing', 'update_one',
             {'queue': 'foo', 'pattern': '*',
              'routing_key': 'foo', 'exchange': 'test_exchange'},
-            {'queue': 'foo', 'pattern': '*',
-             'routing_key': 'foo', 'exchange': 'test_exchange',
-             'expire_at': self.expire_at},
+            {'$set': {
+                'queue': 'foo', 'pattern': '*',
+                'routing_key': 'foo', 'exchange': 'test_exchange',
+                'expire_at': self.expire_at
+            }},
             upsert=True,
         )
 
@@ -450,24 +490,24 @@ class test_mongodb_channel_ttl(BaseMongoDBChannelCase):
         self.channel.queue_delete('foobar')
         self.assert_collection_accessed('messages.queues')
         self.assert_operation_called_with(
-            'queues', 'remove', {'_id': 'foobar'})
+            'queues', 'delete_one', {'_id': 'foobar'})
 
     def test_ensure_indexes(self):
         self.channel._ensure_indexes(self.channel.client)
 
         self.assert_operation_called_with(
-            'messages', 'ensure_index', [('expire_at', 1)],
+            'messages', 'create_index', [('expire_at', 1)],
             expireAfterSeconds=0)
 
         self.assert_operation_called_with(
-            'routing', 'ensure_index', [('expire_at', 1)],
+            'routing', 'create_index', [('expire_at', 1)],
             expireAfterSeconds=0)
 
         self.assert_operation_called_with(
-            'queues', 'ensure_index', [('expire_at', 1)], expireAfterSeconds=0)
+            'queues', 'create_index', [('expire_at', 1)], expireAfterSeconds=0)
 
-    def test_get_expire(self):
-        result = self.channel._get_expire(
+    def test_get_queue_expire(self):
+        result = self.channel._get_queue_expire(
             {'arguments': {'x-expires': 777}}, 'x-expires')
 
         self.channel.client.assert_not_called()
@@ -478,9 +518,15 @@ class test_mongodb_channel_ttl(BaseMongoDBChannelCase):
             '_id': 'docId', 'options': {'arguments': {'x-expires': 777}},
         })
 
-        result = self.channel._get_expire('foobar', 'x-expires')
+        result = self.channel._get_queue_expire('foobar', 'x-expires')
         assert result == self.expire_at
 
+    def test_get_message_expire(self):
+        assert self.channel._get_message_expire({
+            'properties': {'expiration': 777},
+        }) == self.expire_at
+        assert self.channel._get_message_expire({}) is None
+
     def test_update_queues_expire(self):
         self.set_operation_return_value('queues', 'find_one', {
             '_id': 'docId', 'options': {'arguments': {'x-expires': 777}},
@@ -489,16 +535,14 @@ class test_mongodb_channel_ttl(BaseMongoDBChannelCase):
 
         self.assert_collection_accessed('messages.routing', 'messages.queues')
         self.assert_operation_called_with(
-            'routing', 'update',
+            'routing', 'update_many',
             {'queue': 'foobar'},
             {'$set': {'expire_at': self.expire_at}},
-            multi=True,
         )
         self.assert_operation_called_with(
-            'queues', 'update',
+            'queues', 'update_many',
             {'_id': 'foobar'},
             {'$set': {'expire_at': self.expire_at}},
-            multi=True,
         )
 
 
@@ -515,10 +559,19 @@ class test_mongodb_channel_calc_queue_size(BaseMongoDBChannelCase):
     # Tests
 
     def test_size(self):
-        self.set_operation_return_value('messages', 'find.count', 77)
+        self.set_operation_return_value('messages', 'count_documents', 77)
 
         result = self.channel._size('foobar')
 
         self.assert_operation_has_calls('messages', 'find', [])
 
         assert result == 0
+
+
+class test_mongodb_transport(BaseMongoDBChannelCase):
+    def setup(self):
+        self.connection = _create_mock_connection()
+
+    def test_driver_version(self):
+        version = self.connection.transport.driver_version()
+        assert version == pymongo.__version__
